/* Sistemas Integrados TECNISA
   Servios

Nome: ATUSALDO_EMPPED

Descrio:
   Atualiza o saldo do empenho quando o pedido  alterado
Criao:
05/02/07 - Welbert G. Moitinha */


IF EXISTS (SELECT *
           FROM SYSOBJECTS O, SYSUSERS U
           WHERE
              O.NAME = 'ATUSALDO_EMPPED'
              AND TYPE = 'TR'
              AND U.UID = O.UID
              AND U.NAME = USER)
   DROP TRIGGER ATUSALDO_EMPPED;
GO

CREATE TRIGGER ATUSALDO_EMPPED
ON PEDIDO FOR INSERT, UPDATE, DELETE
AS
   /*VARIVEIS DO CURSOR C_CURSOR*/
   DECLARE @V_ID_COMANDO          VARCHAR(1);
   DECLARE @V_CDFILIAL_OLD        VARCHAR(4);
   DECLARE @V_NRPEDIDO_OLD        VARCHAR(10);
   DECLARE @V_CDCLIENTE_OLD       VARCHAR(21);
   DECLARE @V_IDSITPEDIDO_OLD     VARCHAR(1);
   DECLARE @V_NREMPENHO_OLD       VARCHAR(20);
   DECLARE @V_CDFILIAL_NEW        VARCHAR(4);
   DECLARE @V_NRPEDIDO_NEW        VARCHAR(10);
   DECLARE @V_CDCLIENTE_NEW       VARCHAR(21);
   DECLARE @V_IDSITPEDIDO_NEW     VARCHAR(1);
   DECLARE @V_NREMPENHO_NEW       VARCHAR(20);

   DECLARE @V_IDTABPRECO   VARCHAR(1);
   DECLARE @V_IDUTCTRELANP VARCHAR(1);
   DECLARE @V_IDUTCTREMPEN VARCHAR(1);
   DECLARE @V_IDSITPEDIDO  VARCHAR(1);
   DECLARE @V_SALDOGERAL   FLOAT;
   DECLARE @V_VRPEDIDO     FLOAT;
BEGIN
   SET @V_IDTABPRECO   = '';
   SET @V_IDUTCTRELANP = '';
   SET @V_IDUTCTREMPEN = '';
   SET @V_SALDOGERAL   = 0;
   SET @V_VRPEDIDO     = 0;

   DECLARE C_CURSOR CURSOR FOR
      SELECT
         'U' IDCOMANDO,

         I.CDFILIAL,
         I.NRPEDIDO,
         I.CDCLIENTE,
         I.IDSITPEDIDO,
         I. NREMPENHO,
         D.CDFILIAL,
         D.NRPEDIDO,
         D.CDCLIENTE,
         D.IDSITPEDIDO,
         D.NREMPENHO
      FROM
         INSERTED I,
         DELETED D
      WHERE
         I.CDFILIAL  = D.CDFILIAL AND
         I.NRPEDIDO   = D.NRPEDIDO

      UNION ALL

      SELECT
         'I' IDCOMANDO,

         INSERTED.CDFILIAL,
         INSERTED.NRPEDIDO,
         INSERTED.CDCLIENTE,
         INSERTED.IDSITPEDIDO,
         INSERTED.NREMPENHO,
         NULL,
         NULL,
         NULL,
         NULL,
         NULL
      FROM
         INSERTED
      WHERE
         0 = (SELECT COUNT(*) FROM DELETED);


   OPEN C_CURSOR;
   FETCH NEXT FROM C_CURSOR INTO    @V_ID_COMANDO,
                                    @V_CDFILIAL_NEW,
                                    @V_NRPEDIDO_NEW,
                                    @V_CDCLIENTE_NEW,
                                    @V_IDSITPEDIDO_NEW,
                                    @V_NREMPENHO_NEW,
                                    @V_CDFILIAL_OLD,
                                    @V_NRPEDIDO_OLD,
                                    @V_CDCLIENTE_OLD,
                                    @V_IDSITPEDIDO_OLD,
                                    @V_NREMPENHO_OLD;

   SELECT @V_IDTABPRECO = P.IDTABPRECO,
          @V_IDUTCTRELANP = P.IDUTCTRELANP,
             @V_IDUTCTREMPEN = PR.IDUTCTREMPEN
   FROM PARAMPEDIDO P, PARAMGERAL PR
   WHERE
      P.CDFILIAL = ISNULL(@V_CDFILIAL_NEW, @V_CDFILIAL_OLD);


   WHILE (@@FETCH_STATUS = 0) BEGIN

      IF(@V_IDTABPRECO = '1')AND(@V_IDUTCTREMPEN = 'S')BEGIN

         IF (@V_ID_COMANDO = 'U')AND
            (ISNULL( @V_NREMPENHO_OLD, 'M') <> ISNULL(  @V_NREMPENHO_NEW, 'M'))AND
            (@V_IDUTCTRELANP = '2') BEGIN

            SELECT @V_VRPEDIDO = SUM( QTITEMPED* VRPREUNITPED - ISNULL(VRDESITPED,0))
            FROM  ITEMPEDIDO
            WHERE
               @V_CDFILIAL_NEW = CDFILIAL AND
               @V_NRPEDIDO_NEW = NRPEDIDO;

             /*PASSA O VALOR DO PEDIDO PARA O EMPENHO ANTERIOR*/
             IF @V_NREMPENHO_OLD >= '0' BEGIN
               UPDATE EMPENHO
               SET VRSALDOEMP = (@V_VRPEDIDO + VRSALDOEMP)
               WHERE
                  CDCLIENTE =  @V_CDCLIENTE_OLD AND
                  NREMPENHO =  @V_NREMPENHO_OLD;
             END;

             IF @V_NREMPENHO_NEW >= '0' BEGIN
               
               SELECT @V_SALDOGERAL = VRSALDOEMP
               FROM EMPENHO
               WHERE
                 CDCLIENTE = @V_CDCLIENTE_NEW AND
                 NREMPENHO = @V_NREMPENHO_NEW;
               IF(@V_SALDOGERAL >0)AND(@V_SALDOGERAL >=@V_VRPEDIDO)BEGIN

               /* ABATE O VALOR DO PEDIDO NO NOVO EMPENHO */
                  UPDATE EMPENHO
                  SET VRSALDOEMP = (VRSALDOEMP - @V_VRPEDIDO)
                  WHERE
                     CDCLIENTE = @V_CDCLIENTE_NEW AND
                     NREMPENHO = @V_NREMPENHO_NEW;
                                    
                END
                ELSE BEGIN
                   RAISERROR('SALDO_INSU', 16,1);
                END;
             END;
         END
         ELSE BEGIN
            IF( (@V_ID_COMANDO = 'U') AND
              (@V_IDSITPEDIDO_NEW <> @V_IDSITPEDIDO_OLD)AND(@V_NREMPENHO_NEW IS NOT NULL))BEGIN

               SELECT @V_VRPEDIDO = SUM( QTITEMPED* VRPREUNITPED - ISNULL(VRDESITPED,0))
               FROM  ITEMPEDIDO
               WHERE
                  @V_CDFILIAL_NEW = CDFILIAL AND
                  @V_NRPEDIDO_NEW = NRPEDIDO;

               IF( @V_IDSITPEDIDO_NEW = '4') BEGIN
                  UPDATE EMPENHO
                  SET VRSALDOEMP = (@V_VRPEDIDO + VRSALDOEMP)
                  WHERE
                     CDCLIENTE = @V_CDCLIENTE_NEW AND
                     NREMPENHO = @V_NREMPENHO_NEW;
               END
               ELSE BEGIN
                  IF(  @V_IDSITPEDIDO_OLD = '4')AND( @V_IDSITPEDIDO_NEW IN( '1', '2')) BEGIN

                     SELECT @V_SALDOGERAL = VRSALDOEMP
                     FROM EMPENHO
                     WHERE
                        CDCLIENTE = @V_CDCLIENTE_NEW AND
                        NREMPENHO = @V_NREMPENHO_NEW;


                     IF(@V_SALDOGERAL >0)AND(@V_SALDOGERAL >=@V_VRPEDIDO)BEGIN
                        UPDATE EMPENHO
                        SET VRSALDOEMP = (VRSALDOEMP - @V_VRPEDIDO)
                        WHERE
                           CDCLIENTE = @V_CDCLIENTE_NEW AND
                           NREMPENHO = @V_NREMPENHO_NEW;
                     END
                     ELSE BEGIN
                        RAISERROR('SALDO_INSU', 16,1);
                     END;
                  END;
               END;
            END;
         END;
      END;

      FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
                                    @V_CDFILIAL_NEW,
                                    @V_NRPEDIDO_NEW,
                                    @V_CDCLIENTE_NEW,
                                    @V_IDSITPEDIDO_NEW,
                                    @V_NREMPENHO_NEW,
                                    @V_CDFILIAL_OLD,
                                    @V_NRPEDIDO_OLD,
                                    @V_CDCLIENTE_OLD,
                                    @V_IDSITPEDIDO_OLD,
                                    @V_NREMPENHO_OLD;

   END;
   CLOSE      C_CURSOR;
   DEALLOCATE C_CURSOR;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'ATUSALDO_EMPPED','5.05.015';
GO
